"use strict";
// tslint:disable max-classes-per-file
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
const callsites_1 = __importDefault(require("callsites"));
const events_1 = __importDefault(require("events"));
const path = __importStar(require("path"));
function initWorkerThreadsWorker() {
    const NativeWorker = require("worker_threads").Worker;
    class Worker extends NativeWorker {
        constructor(scriptPath) {
            const callerPath = callsites_1.default().find(callsite => {
                const filename = callsite.getFileName();
                return Boolean(filename && !filename.match(/\/worker_threads\//) && !filename.match(/\/master\//));
            }).getFileName();
            const workerFilePath = path.join(path.dirname(callerPath), scriptPath);
            super(require.resolve(workerFilePath), [], { esm: true });
            this.mappedEventListeners = new WeakMap();
        }
        addEventListener(eventName, rawListener) {
            const listener = (message) => {
                rawListener({ data: message });
            };
            this.mappedEventListeners.set(rawListener, listener);
            this.on(eventName, listener);
        }
        removeEventListener(eventName, rawListener) {
            const listener = this.mappedEventListeners.get(rawListener) || rawListener;
            this.off(eventName, listener);
        }
    }
    return Worker;
}
function initTinyWorker() {
    const TinyWorker = require("tiny-worker");
    class Worker extends TinyWorker {
        constructor(scriptPath) {
            const callerPath = callsites_1.default().find(callsite => {
                const filename = callsite.getFileName();
                return Boolean(filename && !filename.match(/\/node_modules\/tiny-worker\//) && !filename.match(/\/master\//));
            }).getFileName();
            super(path.join(path.dirname(callerPath), scriptPath), [], { esm: true });
            this.emitter = new events_1.default();
            this.onerror = (error) => this.emitter.emit("error", error);
            this.onmessage = (message) => this.emitter.emit("message", message);
        }
        addEventListener(eventName, listener) {
            this.emitter.addListener(eventName, listener);
        }
        removeEventListener(eventName, listener) {
            this.emitter.removeListener(eventName, listener);
        }
    }
    return Worker;
}
function selectWorkerImplementation() {
    try {
        return initWorkerThreadsWorker();
    }
    catch (error) {
        // tslint:disable-next-line no-console
        console.debug("Node worker_threads not available. Trying to fall back to tiny-worker polyfill...");
        return initTinyWorker();
    }
}
module.exports = {
    selectWorkerImplementation
};
